@@ -15,13 +15,17 @@
 -- Parameters are used both for setting up expectations and for verification.
 module Test.MockCat.Param
   ( Param(..),
-    WrapParam(..),
+    EqParams(..),
+    WrapResult(wrapResult),
     value,
     param,
     ConsGen(..),
     MockSpec(..),
     expect,
     expect_,
+    ToParamParam(..),
+    ToParamArg(..),
+    Normalize,
     any,
     ArgsOf,
     ProjectionArgs,
@@ -30,8 +34,7 @@ module Test.MockCat.Param
     ProjectionReturn,
     projReturn,
     returnValue,
-    Normalize,
-    ToParamArg(..)
+
   )
 where
 
@@ -63,55 +66,54 @@ data Param v where
   -- | A parameter that wraps a value without Eq or Show constraints.
   ValueWrapper :: v -> String -> Param v
 
--- | Class for wrapping raw values into Param.
--- For types with Show and Eq, it uses ExpectValue to enable comparison and display.
--- For other types, it uses ValueWrapper.
-class WrapParam a where
-  wrap :: a -> Param a
 
-instance {-# OVERLAPPING #-} WrapParam String where
-  wrap s = ExpectValue s (show s)
+-- | Class for wrapping raw values into Param for results.
+-- Does not require Eq or Show, but will use them if available for better display.
+class WrapResult a where
+  wrapResult :: a -> Param a
 
-instance {-# OVERLAPPING #-} WrapParam Int where
-  wrap v = ExpectValue v (show v)
+instance {-# OVERLAPPING #-} WrapResult String where
+  wrapResult s = ExpectValue s (show s)
 
-instance {-# OVERLAPPING #-} WrapParam Integer where
-  wrap v = ExpectValue v (show v)
+instance {-# OVERLAPPING #-} WrapResult Int where
+  wrapResult v = ExpectValue v (show v)
 
-instance {-# OVERLAPPING #-} WrapParam Bool where
-  wrap v = ExpectValue v (show v)
+instance {-# OVERLAPPING #-} WrapResult Integer where
+  wrapResult v = ExpectValue v (show v)
 
-instance {-# OVERLAPPING #-} WrapParam Double where
-  wrap v = ExpectValue v (show v)
+instance {-# OVERLAPPING #-} WrapResult Bool where
+  wrapResult v = ExpectValue v (show v)
 
-instance {-# OVERLAPPING #-} WrapParam Float where
-  wrap v = ExpectValue v (show v)
+instance {-# OVERLAPPING #-} WrapResult Double where
+  wrapResult v = ExpectValue v (show v)
 
-instance {-# OVERLAPPING #-} WrapParam Char where
-  wrap v = ExpectValue v (show v)
+instance {-# OVERLAPPING #-} WrapResult Float where
+  wrapResult v = ExpectValue v (show v)
 
-instance {-# OVERLAPPING #-} WrapParam T.Text where
-  wrap v = ExpectValue v (show v)
+instance {-# OVERLAPPING #-} WrapResult Char where
+  wrapResult v = ExpectValue v (show v)
 
-instance {-# OVERLAPPING #-} (Show a, Eq a) => WrapParam [a] where
-  wrap v = ExpectValue v (show v)
+instance {-# OVERLAPPING #-} WrapResult T.Text where
+  wrapResult v = ExpectValue v (show v)
 
-instance {-# OVERLAPPING #-} (Show a, Eq a) => WrapParam (Maybe a) where
-  wrap v = ExpectValue v (show v)
+instance {-# OVERLAPPABLE #-} (Show a, Eq a) => WrapResult (Maybe a) where
+  wrapResult v = ExpectValue v (show v)
 
-instance {-# OVERLAPPABLE #-} WrapParam a where
-  wrap v = ValueWrapper v "ValueWrapper"
+instance {-# OVERLAPPABLE #-} WrapResult a where
+  wrapResult v = ValueWrapper v "ValueWrapper"
 
 instance Eq (Param a) where
-  (ExpectValue a _) == (ExpectValue b _) = a == b
-  (ExpectValue a _) == (ExpectCondition m2 _) = m2 a
-  (ExpectCondition m1 _) == (ExpectValue b _) = m1 b
-  (ExpectCondition _ l1) == (ExpectCondition _ l2) = l1 == l2
-  ValueWrapper a _ == ExpectCondition m _ = m a
-  ExpectCondition m _ == ValueWrapper a _ = m a
+  ExpectValue a _ == ExpectValue b _ = a == b
   ExpectValue a _ == ValueWrapper b _ = a == b
+  ExpectValue a _ == ExpectCondition m _ = m a
+  ValueWrapper a _ == ValueWrapper b _ = compareFunction a b
   ValueWrapper a _ == ExpectValue b _ = a == b
-  ValueWrapper _ _ == ValueWrapper _ _ = False
+  ValueWrapper a _ == ExpectCondition m _ = m a
+  ExpectCondition m _ == ExpectValue b _ = m b
+  ExpectCondition m _ == ValueWrapper a _ = m a
+  ExpectCondition _ "any" == ExpectCondition _ _ = True
+  ExpectCondition _ _ == ExpectCondition _ "any" = True
+  ExpectCondition _ l1 == ExpectCondition _ l2 = l1 == l2
 
 instance Show (Param v) where
   show (ExpectValue _ l) = l
@@ -138,18 +140,75 @@ type family Normalize a where
 class ToParamArg a where
   toParamArg :: a -> Normalize a
 
+
 instance {-# OVERLAPPING #-} (Typeable (a -> b)) => ToParamArg (a -> b) where
-  toParamArg f = ExpectCondition (compareFunction f) (showFunction f)
+  toParamArg f = ValueWrapper f (showFunction f)
+
+class EqParams a where
+  eqParams :: a -> a -> Bool
+
+instance {-# OVERLAPPING #-} EqParams (Param a) where
+  eqParams = (==)
+
+instance {-# OVERLAPPING #-} (EqParams a, EqParams b) => EqParams (a :> b) where
+  (a1 :> b1) `eqParams` (a2 :> b2) = a1 `eqParams` a2 && b1 `eqParams` b2
+
+instance {-# OVERLAPPING #-} EqParams Head where
+  eqParams _ _ = True
+
+instance EqParams () where
+  eqParams _ _ = True
 
-instance {-# OVERLAPPING #-} ToParamArg (Param a) where
-  toParamArg = id
 
-instance {-# OVERLAPPABLE #-} (Normalize a ~ Param a, WrapParam a) => ToParamArg a where
-  toParamArg = wrap
 
 instance {-# OVERLAPPING #-} ToParamArg Head where
   toParamArg = id
 
+instance {-# OVERLAPPING #-} ToParamArg Int where
+  toParamArg v = ExpectValue v (show v)
+
+instance {-# OVERLAPPING #-} ToParamArg Integer where
+  toParamArg v = ExpectValue v (show v)
+
+instance {-# OVERLAPPING #-} ToParamArg Double where
+  toParamArg v = ExpectValue v (show v)
+
+instance {-# OVERLAPPING #-} ToParamArg Float where
+  toParamArg v = ExpectValue v (show v)
+
+instance {-# OVERLAPPING #-} ToParamArg Bool where
+  toParamArg v = ExpectValue v (show v)
+
+instance {-# OVERLAPPING #-} ToParamArg Char where
+  toParamArg v = ExpectValue v (show v)
+
+instance {-# OVERLAPPING #-} ToParamArg Word where
+  toParamArg v = ExpectValue v (show v)
+
+instance {-# OVERLAPPING #-} ToParamArg T.Text where
+  toParamArg v = ExpectValue v (show v)
+
+instance {-# OVERLAPPING #-} (Show a, Eq a) => ToParamArg [a] where
+  toParamArg v = ExpectValue v (show v)
+
+instance {-# OVERLAPPING #-} (Show a, Eq a) => ToParamArg (Maybe a) where
+  toParamArg v = ExpectValue v (show v)
+
+instance {-# OVERLAPPABLE #-} (Normalize a ~ Param a, Typeable a, Show a) => ToParamArg a where
+  toParamArg v = ValueWrapper v (show v)
+
+class ToParamParam a where
+  toParamParam :: a -> Normalize a
+
+instance {-# OVERLAPPING #-} ToParamParam (Param a) where
+  toParamParam = id
+
+instance {-# OVERLAPPING #-} (Typeable (a -> b)) => ToParamParam (a -> b) where
+  toParamParam f = ValueWrapper f (showFunction f)
+
+instance {-# OVERLAPPABLE #-} (Normalize a ~ Param a, Show a, Eq a) => ToParamParam a where
+  toParamParam v = ExpectValue v (show v)
+
 class ToParamResult b where
   toParamResult :: b -> Normalize b
 
@@ -159,15 +218,15 @@ instance {-# OVERLAPPING #-} ToParamResult (Param a) where
 instance {-# OVERLAPPING #-} ToParamResult (a :> b) where
   toParamResult = id
 
-instance {-# OVERLAPPABLE #-} (Normalize b ~ Param b, WrapParam b) => ToParamResult b where
-  toParamResult = wrap
+instance {-# OVERLAPPABLE #-} (Normalize b ~ Param b, WrapResult b) => ToParamResult b where
+  toParamResult = wrapResult
 
 class ConsGen a b where
   (~>) :: a -> b -> Normalize a :> Normalize b
 
 -- | Instance for chaining parameters
-instance (ToParamArg a, ToParamResult b) => ConsGen a b where
-  (~>) a b = (:>) (toParamArg a) (toParamResult b)
+instance (ToParamParam a, ToParamResult b) => ConsGen a b where
+  a ~> b = toParamParam a :> toParamResult b
 
 -- | Make a parameter to which any value is expected to apply.
 --   Use with type application to specify the type: @any \@String@
