@@ -12,16 +12,15 @@
 {-# OPTIONS_GHC -Wno-orphans #-}
 {-# OPTIONS_GHC -Wno-name-shadowing #-}
 {-# OPTIONS_GHC -Wno-missing-export-lists #-}
-{-# OPTIONS_GHC -Wno-unused-do-bind #-}
 
 module Test.MockCat.PartialMockTHSpec (spec) where
 
 import Test.MockCat
 import qualified Test.MockCat.PartialMockCommonSpec as PartialMockCommonSpec
 import Test.MockCat.SharedSpecDefs
-import Test.MockCat.Impl ()
 import Prelude hiding (readFile, writeFile)
-import Test.Hspec (Spec)
+import Test.Hspec (Spec, describe)
+import Test.MockCat.Impl ()
 
 instance UserInputGetter IO where
   getInput = getLine
@@ -40,7 +39,8 @@ makePartialMock [t|ExplicitlyReturnMonadicValuesPartialTest|]
 makePartialMock [t|FinderNoImplicit|]
 
 spec :: Spec
-spec = PartialMockCommonSpec.spec deps (program "input.txt" "output.text")
+spec = describe "PartialMockTHSpec" $
+  PartialMockCommonSpec.spec deps (program "input.txt" "output.text")
   where
     deps = PartialMockCommonSpec.PartialMockDeps
       { PartialMockCommonSpec._getInput = Test.MockCat.PartialMockTHSpec._getInput
